package com.example.cpspowerprobe;

import java.util.Timer;
import java.util.TimerTask;

import android.graphics.Color;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Handler.Callback;
import android.os.Message;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.text.SpannableString;
import android.text.method.LinkMovementMethod;
import android.text.util.Linkify;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

/*
import com.androidplot.Plot;
import com.androidplot.Series;
import com.androidplot.xy.*;
*/

public class MainActivity extends Activity {	
	
	private static final String ACTION_USB_PERMISSION =
		    "com.android.example.USB_PERMISSION";
							 
	// UI Elements	
	private TextView mTxtAC = null;
	private TextView mTxtDC =  null;
	private TextView mTxtTemp = null;
	private TextView mTxtMin = null;
	private TextView mTxtMax = null;
	private TextView mTxtStatus = null;
	private ProgressBar mPrgDC = null;
	private ProgressBar mPrgAC = null;
	private boolean mFilter = false;
	
    // UI Elements -- Calibratin Dialog
    private EditText mTxtAcOffset = null;
    private EditText mTxtDcOffset = null;
    private EditText mTxtAcScale = null;
    private EditText mTxtDcScale = null;
    private EditText mTxtAcScale2 = null;
    private EditText mTxtDcScale2 = null;
            
	private Timer mTimer = null;
	private Probe mProbe = null;
	private Probe.CPS_probe_data mProbeData = null;
	private Probe.CPS_calibration_buffer mCalBuf = null;
	private AlertDialog mDlgAbout;
	private AlertDialog mDlgDevInfo;
	private AlertDialog mDlgZero;
	private Dialog mDlgCal;
	private IntentFilter mUsbFilter = null; 	
			
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);                

        // UI Elements        
        mTxtAC = (TextView)findViewById(R.id.txtAC);        
        mTxtDC = (TextView)findViewById(R.id.txtDC);
        mTxtTemp = (TextView)findViewById(R.id.txtTemp);
        mTxtMin = (TextView)findViewById(R.id.txtMin);
        mTxtMax = (TextView)findViewById(R.id.txtMax);
        mTxtStatus = (TextView)findViewById(R.id.txtStatus);
        mPrgDC = (ProgressBar)findViewById(R.id.prgDC);
        mPrgAC = (ProgressBar)findViewById(R.id.prgAC);  
        
        mUsbFilter = new IntentFilter();        
        mUsbFilter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
        mUsbFilter.addAction(ACTION_USB_PERMISSION);
        registerReceiver(mUsbReceiver, mUsbFilter);  
        mProbe = new Probe();                                                     
                              
        final SpannableString s = new SpannableString(getResources().getText(R.string.about_msg));
        Linkify.addLinks(s, Linkify.WEB_URLS);
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder
        	.setTitle("CPS Power Probe")
        	.setIcon(android.R.drawable.ic_dialog_info)
        	.setMessage(s)
			.setCancelable(true)
		    .setPositiveButton("OK",new DialogInterface.OnClickListener() {
				@Override
				public void onClick(DialogInterface dialog,int id) {
					dialog.cancel();
				}
			  });
        mDlgAbout = builder.create();
        builder.setMessage("");
        mDlgDevInfo = builder.create();
        builder.setTitle("Zero Probe");
        builder.setPositiveButton("Zero",new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog,int id) {
				if (mProbe.GetConnStatus() == Probe.PROBE_STS_NOTCONNECTED) {
					Toast.makeText(getApplicationContext(), "Probe not connected", Toast.LENGTH_SHORT).show();
				} else {
					mProbe.SetZeros();
					Toast.makeText(getApplicationContext(), "Zeros Set", Toast.LENGTH_SHORT).show();
				}				
				dialog.cancel();				
			}
		  });
        builder.setNeutralButton("Clear",new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog,int id) {
				mProbe.ClearZeros();
				dialog.cancel();
				Toast.makeText(getApplicationContext(), "Zeros Cleared", Toast.LENGTH_SHORT).show();
			}
		  });
        builder.setNegativeButton("Cancel",new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog,int id) {				
				dialog.cancel();				
			}
		  });
        mDlgZero = builder.create();

        // Calibration Dialog
        mDlgCal = new Dialog(this);
        mDlgCal.requestWindowFeature(Window.FEATURE_NO_TITLE);
        mDlgCal.setContentView(R.layout.cal_dialog_layout);
        mDlgCal.setCancelable(true);        
        mDlgCal.findViewById(R.id.btnCalRead).setOnClickListener(Cal_OnClickListener);
        mDlgCal.findViewById(R.id.btnCalWrite).setOnClickListener(Cal_OnClickListener);
        mDlgCal.findViewById(R.id.btnCalDefault).setOnClickListener(Cal_OnClickListener);
        mDlgCal.findViewById(R.id.btnCalOk).setOnClickListener(Cal_OnClickListener);        
        mTxtAcOffset = (EditText) mDlgCal.findViewById(R.id.txtAcOffset);
        mTxtDcOffset = (EditText) mDlgCal.findViewById(R.id.txtDcOffset);
        mTxtAcScale = (EditText) mDlgCal.findViewById(R.id.txtAcScale);
        mTxtDcScale = (EditText) mDlgCal.findViewById(R.id.txtDcScale);
        mTxtAcScale2 = (EditText) mDlgCal.findViewById(R.id.txtAcScale2);
        mTxtDcScale2 = (EditText) mDlgCal.findViewById(R.id.txtDcScale2);
    }

    public void onDestroy() {
    	unregisterReceiver(mUsbReceiver);
    	super.onDestroy();
    
    }
    
    // button handlers for calibration dialog
    final OnClickListener Cal_OnClickListener = new OnClickListener() {
	    public void onClick(final View v) {
	    	switch (v.getId()) {
	    		case  R.id.btnCalOk:
	    			mDlgCal.hide();
	    			break;
	    		case  R.id.btnCalDefault:
	    			mCalBuf.ac_offset = 0;
	    			mCalBuf.ac_scale = 1;
	    			mCalBuf.dc_offset = 0;
	    			mCalBuf.dc_scale = 1;
	    			mCalBuf.dc_scale2 = 0;
	    			mCalBuf.dc_scale3 = 0;
	    			mCalBuf.ac_scale2 = 0;
	    			mCalBuf.ac_scale3 = 0;
	    			updateCalDisplay();
	    			break;
	    		case  R.id.btnCalRead:	    			
	    			mCalBuf = mProbe.GetCalibration();
	    			updateCalDisplay();	    			    			
	    			break;
	    		case  R.id.btnCalWrite:
	    			mCalBuf.ac_offset = Float.valueOf(mTxtAcOffset.getText().toString());
	    			mCalBuf.ac_scale = Float.valueOf(mTxtAcScale.getText().toString());
	    			mCalBuf.ac_scale2 = Float.valueOf(mTxtAcScale2.getText().toString());
	    			mCalBuf.dc_offset = Float.valueOf(mTxtDcOffset.getText().toString());
	    			mCalBuf.dc_scale = Float.valueOf(mTxtDcScale.getText().toString());
	    			mCalBuf.dc_scale2 = Float.valueOf(mTxtDcScale2.getText().toString());
	    			if (mCalBuf.checkValues()) {
	    				mProbe.SetCalibration(mCalBuf);
	    				Log.d("calibration values set", "true");
	    				Toast.makeText(getApplicationContext(), "Calibration values written successfully", Toast.LENGTH_LONG).show();
	    			} else {
	    				Toast.makeText(getApplicationContext(), "Calibration values out of range - Scale must be between -2 and 2", Toast.LENGTH_LONG).show();
	    				Log.d("calibration values set", "false");
	    			}
	    			break;    		
	    	}
	    }		
    };
    
    private void updateCalDisplay() {
    	mTxtAcOffset.setText(String.format("%.6f", mCalBuf.ac_offset));
		mTxtDcOffset.setText(String.format("%.6f", mCalBuf.dc_offset));
		mTxtAcScale.setText(String.format("%.6f", mCalBuf.ac_scale));
		mTxtDcScale.setText(String.format("%.6f", mCalBuf.dc_scale));	    	
		mTxtAcScale2.setText(String.format("%.6f", mCalBuf.ac_scale2));
		mTxtDcScale2.setText(String.format("%.6f", mCalBuf.dc_scale2));	    	
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.activity_main, menu);
        return true;
    }
    
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
            case R.id.menu_about:            	
                mDlgAbout.show();                
                // make links click-able
                ((TextView)mDlgAbout.findViewById(android.R.id.message)).setMovementMethod(LinkMovementMethod.getInstance());
                return true;     
            case R.id.menu_dev_info:
            	if (mProbe.GetConnStatus() == Probe.PROBE_STS_NOTCONNECTED) {
            		mDlgDevInfo.setMessage("No probe connected");
            	} else {
            		mDlgDevInfo.setMessage("Model:Version\n" + mProbe.GetDeviceInfo() + "\n\nFirmware:\n" + mProbe.GetFWVer());
            	}            	
            	mDlgDevInfo.show();                                
                return true;
            case R.id.menu_zero:
            	mDlgZero.show();                             
                return true;   
            case R.id.menu_filter:
            	mFilter = !item.isChecked();
            	item.setChecked(mFilter);                          
                return true; 
            case R.id.menu_cal:
            	if (mProbe.GetConnStatus() == Probe.PROBE_STS_NOTCONNECTED) {
            		Toast.makeText(getApplicationContext(), "No Probe Connected", Toast.LENGTH_LONG).show();
            	} else {
            		mDlgCal.findViewById(R.id.btnCalRead).performClick(); // update values
            		mDlgCal.show();
            	}
            	return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }
    
    @Override
    protected void onPause() {
    	super.onPause();
    	if (mTimer != null) {
    		mTimer.cancel();
    		mTimer = null;
    	}
    }
       
    @Override
    protected void onNewIntent(Intent intent) {
    	super.onNewIntent(intent);        	
    	setIntent(intent);
    }
    
    @Override
    protected void onResume() {    	
    	super.onResume();
    	// reconnect probe if necessary and restart timer
    	synchronized (this) {    		
	    	if (mProbe.GetConnStatus() == Probe.PROBE_STS_NOTCONNECTED) {
	    		if ( !mProbe.FindProbe(getIntent(), getApplicationContext(), (UsbManager) getSystemService(Context.USB_SERVICE))) {
	    			Toast.makeText(getApplicationContext(), "Probe not found", Toast.LENGTH_SHORT).show();
	    		}
	    	} else {	    		
	    		startTimer();
	    	}
    	}

    }
   
    private void startTimer() {
    	if (mTimer == null && mProbe.GetConnStatus() == Probe.PROBE_STS_CONNECTED) {
	    	mTimer = new Timer();
			mTimer.schedule(new PollTask(), 100, 100);
			mTxtStatus.setText(R.string.connected);
			mTxtStatus.setTextColor(Color.GREEN);
    	}
    }
   
    private int calcProgress(double val, int prgMax, int userMax) {
    	int progress;
    	
    	val = Math.abs(val);
    	if (userMax == 0)
    		return 0;
		if (val > userMax) {
			progress = prgMax; 
		} else {
			progress = (int)((val * prgMax) / userMax);
		}
    	//Log.d("progress", "progress" + progress);
    	return progress;
    }
       
	@SuppressLint("DefaultLocale")
	private String checkBounds(double val, int max) {
    	int min = max * -1;
    	if (val > max)
    		return ">MAX";
    	else if (val < min)
    		return "<MIN";
    	else
    		return String.format("%8.1f", val);
    }
	
	// Handler to update the gui from the polling thread
	final Handler UpdateGuiHandler = new Handler( new Callback() {
		@Override
	    public boolean handleMessage(Message msg) {			
			mPrgDC.setProgress(calcProgress(mProbeData.dc, mPrgDC.getMax(), mProbe.getMaxUserDC()));
			mPrgAC.setProgress(calcProgress(mProbeData.ac, mPrgAC.getMax(), mProbe.getMaxUserAC()));						
			
			mTxtDC.setText(checkBounds(mProbeData.dc, mProbe.getMaxUserDC()));
			mTxtAC.setText(checkBounds(mProbeData.ac, mProbe.getMaxUserAC()));
			mTxtTemp.setText(String.format("Temperature: %8.1f", mProbeData.t));
			mTxtMin.setText(String.format("Min: %s", checkBounds(mProbeData.min, mProbe.getMaxUserDC())));
			mTxtMax.setText(String.format("Max: %s", checkBounds(mProbeData.max, mProbe.getMaxUserDC())));						
			return false;
		}			
	});
	
	// Poll the probe
	class PollTask extends TimerTask{		
		@Override
		public void run() {			
			try {
				mProbeData = mProbe.GetValues(mFilter ? (double) 100 : (double) 1);																												
			} catch (Exception e) {
				return;
			}
									
			if (mProbeData != null) {							
				UpdateGuiHandler.sendEmptyMessage(0);								
			}			
		}
	}
	
	// Handle USB detached event
	BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {
	    @Override
		public void onReceive(Context context, Intent intent) {
	        String action = intent.getAction(); 

	      if (UsbManager.ACTION_USB_DEVICE_DETACHED.equals(action)) {
	            UsbDevice device = (UsbDevice)intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
	            if (device != null) {
	            	if (mTimer != null) {
	            		mTimer.cancel();
	            		mTimer.purge();	            		
	            		mTimer = null;
	            	}
	            	
	            	mProbe.Close();	            	
	                mTxtStatus.setText(R.string.notConnected);
	                mTxtStatus.setTextColor(Color.RED);
	                Toast.makeText(getApplicationContext(), "Probe Disconnected", Toast.LENGTH_LONG).show();
	            }
	        } else if (ACTION_USB_PERMISSION.equals(action)) {
	            synchronized (this) {
	                UsbDevice device = (UsbDevice)intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
	                if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, false)) {
	                    if(device != null){
	                    	UsbManager usbMan = null;
	                    	Context cont = null;	                    		                    	
	                    	usbMan = (UsbManager) getSystemService(Context.USB_SERVICE);	                    	
	                    	cont = getApplicationContext();	                    		        	        		                    		
	                    	mProbe.UsbConnect(device, cont, usbMan);
	                    	startTimer();
	                   }
	                } else {
	                	mProbe.Close();
	                	Toast.makeText(getApplicationContext(), "Probe requires permission", Toast.LENGTH_LONG).show();
	                }
	                
	            }
	        	
	        }

	      	      
	    }
	};
	
 
}
